package com.liangjian.dataplatform.member.controller;

import com.liangjian.dataplatform.member.entity.response.ResultObject;
import com.liangjian.dataplatform.member.entity.token.DynamicParamGetter;
import com.liangjian.dataplatform.member.service.NameServiceMapper;
import com.liangjian.dataplatform.member.util.dynamic.DynamicParams;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * Created by huangziqi on 2019/12/26
 *
 * 试验性的restful风格的API控制器，通过用path variable来控制
 */
@RestController
@RequestMapping("/rest/api/model")
@SuppressWarnings("unchecked")
public class RestfulApiController {

    @Autowired
    protected HttpServletRequest httpServletRequest;
    @Autowired
    protected NameServiceMapper nameServiceMapper;

    @GetMapping("/{model}/{id}")
    public ResultObject<?> getSingleModel(@PathVariable("model") String model, @PathVariable("id") String id) {
        return (ResultObject<?>) nameServiceMapper
                .getDynamicApiService(model)
                .getJpaRepository()
                .findById(id)
                .map(i -> ResultObject.success(i))
                .orElse(ResultObject.error("model not found by id"));
    }

    @GetMapping("/{model}/dynamic/page")
    public Page<?> getPagedModel(@PathVariable("model") String model) {
        var params = DynamicParamGetter.DynamicParams.get();
        return nameServiceMapper.getDynamicApiService(model).queryPageByCondition(params);
    }

    @GetMapping("/{model}/dynamic/list")
    public List<?> getListModel(@PathVariable("model") String model) {
        var params = DynamicParamGetter.DynamicParams.get();
        return nameServiceMapper.getDynamicApiService(model).queryListByCondition(params);
    }

    @PostMapping("/{model}")
    public ResultObject<?> saveModel(@PathVariable("model") String model, @RequestBody String jsonStr) {
        if (nameServiceMapper.getDynamicApiService(model).addModel(jsonStr) ) {
            return ResultObject.success("add success",null);
        } else {
            return ResultObject.error("add error");
        }
    }

    @PutMapping("/{model}")
    public ResultObject<?> updateModel(@PathVariable("model") String model, @RequestBody String jsonStr) {
        if (nameServiceMapper.getDynamicApiService(model).saveModel(jsonStr) ) {
            return ResultObject.success("save success",null);
        } else {
            return ResultObject.error("save error");
        }
    }

    //TODO 后面再想怎么谢更新部分字段
    //@PutMapping("/:model/:id")
    //public ResultObject<?> updateModel(@PathVariable("model") String model, @PathVariable("id") String id) {
    //}

    @DeleteMapping("/{model}/{id}")
    public ResultObject<?> deleteModel(@PathVariable("model") String model, @PathVariable("id") String id) {
        if(nameServiceMapper.getDynamicApiService(model).deleteModel(id)) {
            return ResultObject.success("delete success",null);
        } else {
            return ResultObject.error("delete error");
        }
    }
}
