package com.liangjian.dataplatform.member.service;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.ObjectReader;
import com.liangjian.dataplatform.member.safe.Tuple;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;

/**
 * Created by huangziqi on 2019/12/26
 */
public abstract class DynamicApiService<T> extends DynamicQueryService<T> {

    private static Logger log = LoggerFactory.getLogger(DynamicApiService.class);

    public abstract String getMappedName();

    public abstract Class<T> getModelClass();

    public abstract JpaRepository<T,String> getJpaRepository();

    public DynamicApiService getService() {
        return this;
    }

    private T parseJson(String jsonStr) throws JsonProcessingException, JsonMappingException {
        Class<T> clazz = getModelClass();
        ObjectMapper mapper = new ObjectMapper();
        ObjectReader reader = mapper.readerFor(clazz);
        T model = reader.readValue(jsonStr);
        return model;
    }

    public void onAdd(T model) {
        log.warn(String.format("RestfulApiService for %s doesn't have a onAdd, it may save wired with jpa.", getMappedName()));
    }

    public void onUpdate(T model) {
        log.warn(String.format("RestfulApiService for %s doesn't have a onUpdate, it may save wired with jpa.",getMappedName()));
    }

    public boolean addModel(String jsonStr) {
        try {
            T model = parseJson(jsonStr);
            onAdd(model);
            getJpaRepository().save(model);
            return true;
        } catch (Exception ex) {
            ex.printStackTrace();
            return false;
        }
    }

    public boolean saveModel(String jsonStr) {
        try {
            T model = parseJson(jsonStr);
            onUpdate(model);
            getJpaRepository().save(model);
            return true;
        } catch (Exception ex) {
            ex.printStackTrace();
            return false;
        }
    }

    public boolean deleteModel(String id){
        try {
            getJpaRepository().deleteById(id);
            return true;
        } catch (Exception ex) {
            ex.printStackTrace();
            return false;
        }
    }
}
